@@ -238,7 +238,7 @@ namespace pl::ptrn {
         [[nodiscard]] virtual std::string toString() const {
             auto result = fmt::format("{} {} @ 0x{:X}", this->getTypeName(), this->getVariableName(), this->getOffset());
 
-            return this->formatDisplayValue(result, this->getValue(), true);
+            return this->callUserFormatFunc(this->getValue(), true).value_or(result);
         }
 
         [[nodiscard]] virtual core::Token::Literal getValue() const {
@@ -516,14 +516,14 @@ namespace pl::ptrn {
         }
 
         [[nodiscard]] virtual std::string formatDisplayValue() = 0;
-
-        [[nodiscard]] std::string formatDisplayValue(const std::string &value, const core::Token::Literal &literal, bool fromCast = false) const {
-            if (this->m_cachedDisplayValue != nullptr)
-                return *this->m_cachedDisplayValue;
-
+        
+        /**
+         * @brief Calls an user-defined PL function to format the given literal (pattern), if existing. Else, returns defaultValue
+        */
+        [[nodiscard]] std::optional<std::string> callUserFormatFunc(const core::Token::Literal &literal, bool fromCast = false) const {
             const auto &formatterFunctionName = this->getReadFormatterFunction();
             if (formatterFunctionName.empty())
-                return value;
+                return {};
             else {
                 try {
                     const auto function = this->m_evaluator->findFunction(formatterFunctionName);
@@ -533,23 +533,14 @@ namespace pl::ptrn {
 
                         auto result = function->func(this->m_evaluator, { literal });
                         if (result.has_value()) {
-                            std::string string;
                             if (fromCast && result->isPattern() && result->toPattern()->getTypeName() == this->getTypeName()) {
-                                string = value;
+                                return {};
                             } else {
-                                string = result->toString(true);
+                                return result->toString(true);
                             }
-
-
-                            this->m_cachedDisplayValue = std::make_unique<std::string>(string);
-
-                            return string;
-                        } else {
-                            return "";
                         }
-                    } else {
-                        return "";
                     }
+                    return {};
 
                 } catch (core::err::EvaluatorError::Exception &error) {
                     return error.what();
