@@ -23,18 +23,13 @@
 #include <pl/patterns/pattern_array_dynamic.hpp>
 #include <pl/patterns/pattern_padding.hpp>
 
+#include <exception>
 #include <utility>
 
 namespace pl::core {
 
     std::map<std::string, Token::Literal> Evaluator::getOutVariables() const {
-        std::map<std::string, Token::Literal> result;
-
-        for (const auto &[name, pattern] : this->m_outVariables) {
-            result.insert({ name, pattern->getValue() });
-        }
-
-        return result;
+        return m_outVariableValues;
     }
 
     void Evaluator::setDataSource(u64 baseAddress, size_t dataSize, std::function<void(u64, u8*, size_t)> readerFunction, std::optional<std::function<void(u64, const u8*, size_t)>> writerFunction) {
@@ -48,7 +43,7 @@ namespace pl::core {
         };
 
         if (writerFunction.has_value()) {
-            this->m_writerFunction = [this, writerFunction = std::move(writerFunction.value())](u64 offset, u8* buffer, size_t size) {
+            this->m_writerFunction = [this, writerFunction = std::move(writerFunction.value())](u64 offset, const u8* buffer, size_t size) {
                 this->m_lastWriteAddress = offset;
 
                 writerFunction(offset, buffer, size);
@@ -138,17 +133,37 @@ namespace pl::core {
         this->writeData(byteOffset, &oldValue, writeSize, section);
     }
 
+
+
     bool Evaluator::addBuiltinFunction(const std::string &name, api::FunctionParameterCount numParams, std::vector<Token::Literal> defaultParameters, const api::FunctionCallback &function, bool dangerous) {
         const auto [iter, inserted] = this->m_builtinFunctions.insert({
-            name, {numParams, std::move(defaultParameters), function, dangerous}
+            name, {
+                numParams,
+                std::move(defaultParameters),
+                dangerous ? handleDangerousFunctionCall(name, function) : function
+            }
         });
 
         return inserted;
     }
 
+    api::FunctionCallback Evaluator::handleDangerousFunctionCall(const std::string &functionName, const api::FunctionCallback &function) {
+        return [this, function, functionName](core::Evaluator *, const std::vector<core::Token::Literal> &params) -> std::optional<core::Token::Literal> {
+            if (getDangerousFunctionPermission() != DangerousFunctionPermission::Allow) {
+                dangerousFunctionCalled();
+
+                if (getDangerousFunctionPermission() == DangerousFunctionPermission::Deny) {
+                    err::E0009.throwError(fmt::format("Call to dangerous function '{}' has been denied.", functionName), { });
+                }
+            }
+
+            return function(this, params);
+        };
+    }
+
     bool Evaluator::addCustomFunction(const std::string &name, api::FunctionParameterCount numParams, std::vector<Token::Literal> defaultParameters, const api::FunctionCallback &function) {
         const auto [iter, inserted] = this->m_customFunctions.insert({
-            name, {numParams, std::move(defaultParameters), function, false}
+            name, {numParams, std::move(defaultParameters), function}
         });
 
         return inserted;
@@ -173,7 +188,7 @@ namespace pl::core {
         };
     }
 
-    void Evaluator::createArrayVariable(const std::string &name, ast::ASTNode *type, size_t entryCount, u64 section, bool constant) {
+    void Evaluator::createArrayVariable(const std::string &name, const ast::ASTNode *type, size_t entryCount, u64 section, bool constant) {
         // A variable named _ gets treated as "don't care"
         if (name == "_")
             return;
@@ -181,7 +196,7 @@ namespace pl::core {
         auto &variables = *this->getScope(0).scope;
         for (auto &variable : variables) {
             if (variable->getVariableName() == name) {
-                err::E0003.throwError(fmt::format("Variable with name '{}' already exists in this scope.", name), {}, type);
+                err::E0003.throwError(fmt::format("Variable with name '{}' already exists in this scope.", name), {}, type->getLocation());
             }
         }
 
@@ -189,15 +204,35 @@ namespace pl::core {
         auto typePatterns = type->createPatterns(this);
         auto typePattern = std::move(typePatterns.front());
 
+        typePattern->setConstant(constant);
+
         this->setBitwiseReadOffset(startOffset);
 
         auto pattern = new ptrn::PatternArrayDynamic(this, 0, typePattern->getSize() * entryCount, 0);
 
-        if (section == ptrn::Pattern::HeapSectionId) {
+        if (section == ptrn::Pattern::PatternLocalSectionId) {
+            typePattern->setSection(section);
+            std::vector<std::shared_ptr<ptrn::Pattern>> entries;
+            for (size_t i = 0; i < entryCount; i++) {
+                auto entryPattern = typePattern->clone();
+                entryPattern->setSection(section);
+
+                auto patternLocalAddress = this->m_patternLocalStorage.empty() ? 0 : this->m_patternLocalStorage.rbegin()->first + 1;
+                entryPattern->setOffset(u64(patternLocalAddress) << 32);
+                this->m_patternLocalStorage.insert({ patternLocalAddress, { } });
+                this->m_patternLocalStorage[patternLocalAddress].data.resize(entryPattern->getSize());
+
+                entries.push_back(std::move(entryPattern));
+            }
+            pattern->setEntries(std::move(entries));
+            pattern->setSection(section);
+
+        } else if (section == ptrn::Pattern::HeapSectionId) {
             typePattern->setLocal(true);
             std::vector<std::shared_ptr<ptrn::Pattern>> entries;
             for (size_t i = 0; i < entryCount; i++) {
                 auto entryPattern = typePattern->clone();
+                entryPattern->setLocal(true);
 
                 auto &heap = this->getHeap();
                 entryPattern->setOffset(u64(heap.size()) << 32);
@@ -228,7 +263,7 @@ namespace pl::core {
         variables.push_back(std::unique_ptr<ptrn::Pattern>(pattern));
     }
 
-    static std::optional<std::string> findTypeName(ast::ASTNodeTypeDecl *type) {
+    static std::optional<std::string> findTypeName(const ast::ASTNodeTypeDecl *type) {
 
         const ast::ASTNodeTypeDecl *typeDecl = type;
         while (true) {
@@ -241,7 +276,7 @@ namespace pl::core {
         }
     }
 
-    static ast::ASTNodeBuiltinType* getBuiltinType(ast::ASTNodeTypeDecl *type) {
+    static ast::ASTNodeBuiltinType* getBuiltinType(const ast::ASTNodeTypeDecl *type) {
         const ast::ASTNodeTypeDecl *typeDecl = type;
         while (true) {
             if (auto innerType = dynamic_cast<ast::ASTNodeTypeDecl*>(typeDecl->getType().get()); innerType != nullptr)
@@ -253,9 +288,9 @@ namespace pl::core {
         }
     }
 
-    std::shared_ptr<ptrn::Pattern> Evaluator::createVariable(const std::string &name, ast::ASTNodeTypeDecl *type, const std::optional<Token::Literal> &value, bool outVariable, bool reference, bool templateVariable, bool constant) {
-        auto startPos = this->getReadOffset();
-        ON_SCOPE_EXIT { this->setReadOffset(startPos); };
+    std::shared_ptr<ptrn::Pattern> Evaluator::createVariable(const std::string &name, const ast::ASTNodeTypeDecl *type, const std::optional<Token::Literal> &value, bool outVariable, bool reference, bool templateVariable, bool constant) {
+        auto startPos = this->getBitwiseReadOffset();
+        ON_SCOPE_EXIT { this->setBitwiseReadOffset(startPos); };
 
         // A variable named _ gets treated as "don't care"
         if (name == "_")
@@ -269,7 +304,7 @@ namespace pl::core {
             auto &currScope = this->getScope(0);
             for (auto &variable : *currScope.scope) {
                 if (variable->getVariableName() == name) {
-                    err::E0003.throwError(fmt::format("Variable with name '{}' already exists in this scope.", name), {}, type);
+                    err::E0003.throwError(fmt::format("Variable with name '{}' already exists in this scope.", name), {}, type->getLocation());
                 }
             }
         }
@@ -286,7 +321,7 @@ namespace pl::core {
             } else if (sectionId == ptrn::Pattern::HeapSectionId) {
                 this->getHeap().emplace_back();
             } else {
-                err::E0001.throwError(fmt::format("Attempted to place a variable into section 0x{:X}.", sectionId), {}, type);
+                err::E0001.throwError(fmt::format("Attempted to place a variable into section 0x{:X}.", sectionId), {}, type->getLocation());
             }
         }
 
@@ -317,7 +352,7 @@ namespace pl::core {
                     pattern = (*patternValue)->clone();
             }
             else
-                err::E0003.throwError("Cannot determine type of 'auto' variable.", "Try initializing it directly with a literal.", type);
+                err::E0003.throwError("Cannot determine type of 'auto' variable.", "Try initializing it directly with a literal.", type->getLocation());
         } else {
             if (builtinType != nullptr)
                 pattern = builtinType->createPatterns(this).front();
@@ -327,7 +362,7 @@ namespace pl::core {
                 if (auto typeName = findTypeName(type); typeName.has_value())
                     pattern->setTypeName(typeName.value());
                 else
-                    err::E0003.throwError("Cannot determine type.", "", type);
+                    err::E0003.throwError("Cannot determine type.", "", type->getLocation());
             }
         }
 
@@ -350,9 +385,9 @@ namespace pl::core {
 
         if (outVariable) {
             if (this->isGlobalScope())
-                this->m_outVariables[name] = pattern->clone();
+                this->m_outVariables[name] = pattern;
             else
-                err::E0003.throwError("Out variables can only be declared in the global scope.", {}, type);
+                err::E0003.throwError("Out variables can only be declared in the global scope.", {}, type->getLocation());
         }
 
         if (this->isDebugModeEnabled())
@@ -418,7 +453,7 @@ namespace pl::core {
                     err::E0004.throwError(fmt::format("Cannot cast from type 'string' to type '{}'.", pattern->getTypeName()));
             },
             [&](const std::shared_ptr<ptrn::Pattern>& value) -> Token::Literal {
-                if (value->getTypeName() == pattern->getTypeName())
+                if (value->getTypeName() == pattern->getTypeName() || value->getTypeName().empty())
                     return value;
                 else
                     err::E0004.throwError(fmt::format("Cannot cast from type '{}' to type '{}'.", value->getTypeName(), pattern->getTypeName()));
@@ -472,7 +507,7 @@ namespace pl::core {
         err::E0003.throwError(fmt::format("Cannot find variable '{}' in this scope.", name));
     }
 
-    void Evaluator::setVariable(const std::string &name, const Token::Literal &value) {
+    void Evaluator::setVariable(const std::string &name, const Token::Literal &variableValue) {
         // A variable named _ gets treated as "don't care"
         if (name == "_")
             return;
@@ -484,6 +519,9 @@ namespace pl::core {
                 // If the variable is being set to a pattern, adjust its layout to the real layout as it potentially contains dynamically sized members
                 std::visit(wolv::util::overloaded {
                     [&](ptrn::Pattern * const value) {
+                        if (value->getTypeName().empty())
+                            return;
+
                         if (value->getTypeName() != variablePattern->getTypeName() && !variablePattern->getTypeName().empty())
                             err::E0004.throwError(fmt::format("Cannot cast from type '{}' to type '{}'.", value->getTypeName(), variablePattern->getTypeName()));
 
@@ -511,10 +549,13 @@ namespace pl::core {
                             err::E0004.throwError(fmt::format("Cannot assign value of type 'string' to variable of type '{}'.", variablePattern->getTypeName()));
                     },
                     [](const auto &) {}
-                }, value);
+                }, variableValue);
             } else {
                 std::visit(wolv::util::overloaded {
                     [&](ptrn::Pattern * const value) {
+                        if (value->getTypeName().empty())
+                            return;
+
                         if (value->getTypeName() != variablePattern->getTypeName() && !variablePattern->getTypeName().empty())
                             err::E0004.throwError(fmt::format("Cannot cast from type '{}' to type '{}'.", value->getTypeName(), variablePattern->getTypeName()));
                     },
@@ -523,13 +564,13 @@ namespace pl::core {
                             err::E0004.throwError(fmt::format("Cannot assign string of size {} to variable of size {}.", value.size(), variablePattern->getSize()));
                     },
                     [](const auto &) {}
-                }, value);
+                }, variableValue);
             }
 
             return variablePattern;
         }();
 
-        this->setVariable(pattern, value);
+        this->setVariable(pattern, variableValue);
     }
 
     void Evaluator::changePatternType(std::shared_ptr<ptrn::Pattern> &pattern, std::shared_ptr<ptrn::Pattern> &&newPattern) {
@@ -547,9 +588,9 @@ namespace pl::core {
         pattern->setVariableName(variableName);
     }
 
-    void Evaluator::setVariable(std::shared_ptr<ptrn::Pattern> &pattern, const Token::Literal &value) {
-        auto startPos = this->getReadOffset();
-        ON_SCOPE_EXIT { this->setReadOffset(startPos); };
+    void Evaluator::setVariable(std::shared_ptr<ptrn::Pattern> &pattern, const Token::Literal &variableValue) {
+        auto startPos = this->getBitwiseReadOffset();
+        ON_SCOPE_EXIT { this->setBitwiseReadOffset(startPos); };
 
         if (pattern->isConstant() && pattern->isInitialized())
             err::E0011.throwError(fmt::format("Cannot modify constant variable '{}'.", pattern->getVariableName()));
@@ -563,7 +604,7 @@ namespace pl::core {
                     pattern = patternValue;
                 },
                 [](const auto &) { }
-            }, value);
+            }, variableValue);
             return;
         }
 
@@ -571,7 +612,7 @@ namespace pl::core {
             err::E0003.throwError(fmt::format("Value is too large to place into local variable '{}'.", pattern->getVariableName()));
 
         // Cast values to type given by pattern
-        Token::Literal castedValue = castLiteral(pattern.get(), value);
+        Token::Literal castedValue = castLiteral(pattern.get(), variableValue);
 
         // Write value into variable storage
         {
@@ -881,6 +922,7 @@ namespace pl::core {
         this->m_sectionIdStack.clear();
         this->m_sectionId = 1;
         this->m_outVariables.clear();
+        this->m_outVariableValues.clear();
 
         this->m_customFunctions.clear();
         this->m_patterns.clear();
@@ -903,6 +945,10 @@ namespace pl::core {
             this->m_envVariables.clear();
             this->m_evaluated = true;
             this->m_mainSectionEditsAllowed = false;
+
+            for (const auto &[name, pattern] : this->m_outVariables) {
+                this->m_outVariableValues.insert({ name, pattern->getValue() });
+            }
         };
 
         this->m_currPatternCount = 0;
@@ -1024,9 +1070,8 @@ namespace pl::core {
             }
         } catch (err::EvaluatorError::Exception &e) {
 
-            auto node = e.getUserData();
+            auto location = e.getUserData();
 
-            const auto location = node == nullptr ? Location::Empty() : node->getLocation();
             this->getConsole().setHardError(err::PatternLanguageError(e.format(location),
                 location.line, location.column));
 
@@ -1049,8 +1094,8 @@ namespace pl::core {
             return;
 
         evaluator->handleAbort();
-
-        const auto line = node->getLocation().line;
+        auto temp = node->getLocation().line;
+        const auto line = temp + (temp == 0);
         if (evaluator->m_shouldPauseNextLine && evaluator->m_lastPauseLine != line) {
             evaluator->m_shouldPauseNextLine = false;
             evaluator->m_lastPauseLine = line;
@@ -1062,10 +1107,13 @@ namespace pl::core {
             }
         }
 
-        evaluator->m_callStack.push_back(node);
+        evaluator->m_callStack.push_back(node->clone());
     }
 
     Evaluator::UpdateHandler::~UpdateHandler() {
+        if (evaluator->m_evaluated)
+            return;
+
         // Don't pop scopes if an exception is currently being thrown so we can generate
         // a stack tace
         if (std::uncaught_exceptions() > 0)
@@ -1089,7 +1137,7 @@ namespace pl::core {
         return this->m_lastPauseLine;
     }
 
-    void Evaluator::patternCreated(ptrn::Pattern *pattern) {
+    void Evaluator::patternCreated(const ptrn::Pattern *pattern) {
         this->m_lastPatternAddress = pattern->getOffset();
 
         if (this->m_currPatternCount > this->m_patternLimit && !this->m_evaluated)
@@ -1111,7 +1159,7 @@ namespace pl::core {
         }
     }
 
-    void Evaluator::patternDestroyed(ptrn::Pattern *pattern) {
+    void Evaluator::patternDestroyed(const ptrn::Pattern *pattern) {
         this->m_currPatternCount--;
 
         // Make sure we don't throw an error if we're already in an error state
