@@ -15,6 +15,10 @@
         void setupMacosWindowStyle(GLFWwindow *window, bool borderlessWindowMode);
 
         void enumerateFontsMacos();
+    
+        void macosHandleTitlebarDoubleClickGesture(GLFWwindow *window);
+        bool macosIsWindowBeingResizedByUser(GLFWwindow *window);
+        void macosMarkContentEdited(GLFWwindow *window, bool edited = true);
     }
 
-#endif
\ No newline at end of file
+#endif
